program  drive_sem

! 2DSPEC, a program written by Gaetano Festa & Elise Delavaud
! This version has been biult for parallel computers with the MPI
! Current version 2.0
! Actual revision for publication

! Begin correction in the programming                        19/10/2006
! End correction in the programming 




use sdomain

implicit none
include "mpif.h"
type (domain), target  :: Tdomain
integer :: ntime,i_snap, ierr, n_face_pointed

! ##############  Begin of the program  #######################
call MPI_Init (ierr)
call MPI_Comm_Rank (MPI_COMM_WORLD, Tdomain%Mpi_var%my_rank, ierr)
call MPI_Comm_Size (MPI_COMM_WORLD, Tdomain%Mpi_var%n_proc,  ierr)

if (Tdomain%MPI_var%my_rank == 0) write (*,*) "Define mesh properties"
call read_input (Tdomain)

if (Tdomain%MPI_var%my_rank == 0) write (*,*) "Compute Gauss-Lobatto-Legendre weights and zeroes"
call compute_GLL (Tdomain)

if (Tdomain%MPI_var%my_rank == 0) write (*,*) "Define a global numbering for the collocation points"
call global_numbering (Tdomain)

if (Tdomain%MPI_var%my_rank == 0) write (*,*) "Computing shape functions within thier derivatives"
if  (Tdomain%n_nodes == 4) then
       call shape4(TDomain)   ! Linear interpolation
else if (Tdomain%n_nodes == 8) then
       call shape8(TDomain)  ! Quadratic interpolation
else
    if (Tdomain%MPI_var%my_rank == 0) write (*,*) " Bad number of nodes for hexaedral shape "
    stop
endif

if (Tdomain%logicD%plot_grid) then
     if (Tdomain%MPI_var%my_rank == 0) write (*,*) "Plotting the element mesh"
     call plot_grid (Tdomain)
endif

if (Tdomain%MPI_var%my_rank == 0) write (*,*) " Compute Courant parameter"
call compute_Courant (Tdomain)

if (Tdomain%MPI_var%my_rank == 0) write (*,*) "Attribute PML properties"
call PML_definition (Tdomain)

if (Tdomain%logicD%any_source) then
   if (Tdomain%MPI_var%my_rank == 0) write (*,*) , "Computing point-source parameters and location"
   call SourcePosition(Tdomain)
endif

if (Tdomain%logicD%save_trace ) then
        if (Tdomain%MPI_var%my_rank == 0) write (*,*) "Computing receivers parameters and locations"
        call ReceiverPosition(Tdomain)
endif

  if (Tdomain%MPI_var%my_rank ==0 .and. Tdomain%logicD%super_object) write(*,*) "Define Fault properties"
if (Tdomain%logicD%super_object_local_present) then
    if (Tdomain%n_fault > 0) call define_fault_properties (Tdomain)
endif

if (Tdomain%MPI_var%my_rank == 0) write (*,*) " Allocate fields"
call allocate_domain (Tdomain)

if (Tdomain%MPI_var%my_rank == 0) write (*,*) "Compute tranfer quantities"
call wall_transfer (Tdomain)

if (Tdomain%MPI_var%my_rank == 0) write (*,*) " Compute mass matrix and internal forces coefficients"
call define_arrays (Tdomain)

if (Tdomain%MPI_var%my_rank == 0) write (*,*) " Entering the evolution"
call MPI_Barrier (MPI_COMM_WORLD, ierr)
Tdomain%TimeD%rtime = 0
if (Tdomain%logicD%save_snapshots .or. Tdomain%logicD%save_deformation) Tdomain%timeD%nsnap = Tdomain%TimeD%time_snapshots / Tdomain%TimeD%dtmin
i_snap =1;

do ntime= 0, Tdomain%TimeD%NtimeMax-1
    call Newmark (Tdomain)
    Tdomain%TimeD%rtime = Tdomain%TimeD%rtime + Tdomain%TimeD%dtmin
    if (Tdomain%logicD%save_snapshots .or. Tdomain%logicD%save_deformation)  i_snap = mod (ntime+1, Tdomain%TimeD%nsnap)
    if (i_snap == 0 ) then
         if (Tdomain%MPI_var%my_rank == 0) write (*,*) "Iteration number    ", ntime
          if (Tdomain%logicD%save_snapshots) call savefield (Tdomain,ntime)
	  if (Tdomain%logicD%save_deformation) then
	                   call save_vorticity (Tdomain,ntime)
			   call save_deformation (Tdomain, ntime)
	  endif
	  if (Tdomain%logicD%save_fault_trace) call save_fault_trace (Tdomain, ntime)
    endif

    if (Tdomain%logicD%save_trace) call save_trace(Tdomain, ntime)

enddo

if (Tdomain%logicD%save_trace) call dump_trace(Tdomain)
call MPI_Finalize  (ierr)
write (*,*) "Work in progress... more functions to come"
 stop
 end program
 
