subroutine shape4(Tdomain)

use sdomain

implicit none

type(domain),target, intent (INOUT) :: Tdomain


! local variables

integer :: i_aus,n, mat,ngllx,ngllz,i,j,ipoint, ngll, nv, Face_UP, nv2

real :: x0,x1,x2,x3,z0,z1,z2,z3,xi,eta,xp,zp, Jac, ds_local
real :: normal_0, normal_1, normalization
real, dimension (0:1,0:1) :: LocInvGrad
real, dimension (:,:), allocatable :: Store_normal
real, dimension (:), pointer :: GLLc_face

! Modified by Gaetano Festa, 26/05/05
!---------------------------------------------------------------------------------------------------------------
! Shape functions are derived from "Finite Elements and Approximations"
! by Zienkiewicz, O.C. and Morgan, K.
! John Wiley and Sons, 1983
! --------------------------------------------------------------------------------------------------------------

allocate (Tdomain%GlobCoord(0:1,0:Tdomain%n_glob_points-1))

do n = 0,Tdomain%n_elem - 1
   i_aus = Tdomain%specel(n)%Control_Nodes(0);  x0 = Tdomain%Coord_Nodes(0,i_aus);  z0 = Tdomain%Coord_Nodes(1,i_aus)
   i_aus = Tdomain%specel(n)%Control_Nodes(1);  x1 = Tdomain%Coord_Nodes(0,i_aus);  z1 = Tdomain%Coord_Nodes(1,i_aus)
   i_aus = Tdomain%specel(n)%Control_Nodes(2);  x2 = Tdomain%Coord_Nodes(0,i_aus);  z2 = Tdomain%Coord_Nodes(1,i_aus)
   i_aus = Tdomain%specel(n)%Control_Nodes(3);  x3 = Tdomain%Coord_Nodes(0,i_aus);  z3 = Tdomain%Coord_Nodes(1,i_aus)

   mat = Tdomain%specel(n)%mat_index
   ngllx = Tdomain%specel(n)%ngllx
   ngllz = Tdomain%specel(n)%ngllz

   allocate (Tdomain%specel(n)%Jacob(0:ngllx-1,0:ngllz-1) ) 
   allocate (Tdomain%specel(n)%InvGrad(0:ngllx-1,0:ngllz-1,0:1,0:1) ) 

    do j = 0,ngllz - 1

       eta =   Tdomain%sSubdomain(mat)%GLLcz (j)        
 
       do i = 0,ngllx - 1

          xi = Tdomain%sSubdomain(mat)%GLLcx (i)

          ipoint = Tdomain%specel(n)%Iglobnum(i,j) 
 
          xp = 0.25 * ( x0 * (1.-xi)*(1.-eta) + x1 * (1.+xi)*(1.-eta) + x2 * (1.+xi)*(1.+eta) + x3 * (1.-xi)*(1.+eta) )
          zp = 0.25 * ( z0 * (1.-xi)*(1.-eta) + z1 * (1.+xi)*(1.-eta) + z2 * (1.+xi)*(1.+eta) + z3 * (1.-xi)*(1.+eta) )

          Tdomain%GlobCoord (0,ipoint) = xp;   Tdomain%GlobCoord (1,ipoint) = zp
     
!         Computation of the derivative matrix, dx_(jj)/dxi_(ii) 

           LocInvGrad(0,0) = 0.25 * ( (x1-x0) * (1-eta) + (x2-x3) * (1+eta) )
           LocInvGrad(1,0) = 0.25 * ( (x3-x0) * (1-xi) + (x2-x1) * (1+xi) )
           LocInvGrad(0,1) = 0.25 * ( (z1-z0) * (1-eta) + (z2-z3) * (1+eta) )
           LocInvGrad(1,1) = 0.25 * ( (z3-z0) * (1-xi) + (z2-z1) * (1+xi) )

           call invert2 (LocInvGrad, Jac) 

           Tdomain%specel(n)%InvGrad (i,j,0:1,0:1)  = LocInvGrad (0:1,0:1)
 
           Tdomain%specel(n)%Jacob (i,j) = Jac
       enddo
   enddo
enddo

if (Tdomain%logicD%super_object_local_present) then
   do n = 0, Tdomain%n_fault-1
        do  j = 0, Tdomain%sFault(n)%n_face-1
	     ngll = Tdomain%sFault(n)%fFace(j)%ngll
	     Face_up = Tdomain%sFault(n)%fFace(j)%Face_UP
	     nv2 = Tdomain%sFace(Face_up)%Near_element(0)
	     mat = Tdomain%specel(nv2)%mat_index
             if (Tdomain%specel(nv2)%near_face(0) == Face_up .or. Tdomain%specel(nv2)%near_face(2) == face_up) then 
        	     GLLc_face => Tdomain%sSubdomain(mat)%GLLcx 
	    else
        	     GLLc_face => Tdomain%sSubdomain(mat)%GLLcz
	     endif

	     if (Tdomain%specel(nv2)%near_face(0) == Face_up) then
                   i_aus = Tdomain%specel(nv2)%Control_Nodes(0);  x0 = Tdomain%Coord_Nodes(0,i_aus);  z0 = Tdomain%Coord_Nodes(1,i_aus)
                   i_aus = Tdomain%specel(nv2)%Control_Nodes(1);  x1 = Tdomain%Coord_Nodes(0,i_aus);  z1 = Tdomain%Coord_Nodes(1,i_aus)
             else if (Tdomain%specel(nv2)%near_face(1) == Face_up) then
                   i_aus = Tdomain%specel(nv2)%Control_Nodes(1);  x0 = Tdomain%Coord_Nodes(0,i_aus);  z0 = Tdomain%Coord_Nodes(1,i_aus)
                   i_aus = Tdomain%specel(nv2)%Control_Nodes(2);  x1 = Tdomain%Coord_Nodes(0,i_aus);  z1 = Tdomain%Coord_Nodes(1,i_aus)
             else if (Tdomain%specel(nv2)%near_face(2) == Face_up) then
                   i_aus = Tdomain%specel(nv2)%Control_Nodes(2);  x0 = Tdomain%Coord_Nodes(0,i_aus);  z0 = Tdomain%Coord_Nodes(1,i_aus)
                   i_aus = Tdomain%specel(nv2)%Control_Nodes(3);  x1 = Tdomain%Coord_Nodes(0,i_aus);  z1 = Tdomain%Coord_Nodes(1,i_aus)
	     else if (Tdomain%specel(nv2)%near_face(1) == Face_up) then
                   i_aus = Tdomain%specel(nv2)%Control_Nodes(3);  x0 = Tdomain%Coord_Nodes(0,i_aus);  z0 = Tdomain%Coord_Nodes(1,i_aus)
                   i_aus = Tdomain%specel(nv2)%Control_Nodes(0);  x1 = Tdomain%Coord_Nodes(0,i_aus);  z1 = Tdomain%Coord_Nodes(1,i_aus)
	     endif

	     allocate (Tdomain%sFault(n)%fFace(j)%ds(0:ngll-1))
	     allocate (Tdomain%sFault(n)%fFace(j)%normal(0:ngll-1,0:1))

	     ds_local =  (x1-x0)**2 + (z1-z0)**2
	     ds_local =  Sqrt(ds_local)
	     normal_0 = (z1 - z0)/ds_local; normal_1 = (x0-x1)/ds_local

! we define the normal for the Face_DOWN
            normal_0 = - normal_0; normal_1 = - normal_1

	     Tdomain%sFault(n)%fFace(j)%X_Vertex(0) = x0; Tdomain%sFault(n)%fFace(j)%X_Vertex(1) = x1
	     Tdomain%sFault(n)%fFace(j)%Z_Vertex(0) = z0; Tdomain%sFault(n)%fFace(j)%Z_Vertex(1) = z1
	     Tdomain%sFault(n)%fFace(j)%ds(:) = 0.5 * ds_local
	     Tdomain%sFault(n)%fFace(j)%normal(0:ngll-1,0) = normal_0; Tdomain%sFault(n)%fFace(j)%normal(0:ngll-1,1) = normal_1
         enddo

         do j = 0, Tdomain%sFault(n)%n_vertex-1
	     Tdomain%sFault(n)%fVertex(j)%normal(0:1) = 0
	 enddo

	 do j = 0, Tdomain%sFault(n)%n_face-1
	     ngll = Tdomain%sFault(n)%fFace(j)%ngll
	     nv = Tdomain%sFault(n)%fFace(j)%Face_To_Vertex(0)
	     Tdomain%sFault(n)%fVertex(nv)%Normal(0:1) = Tdomain%sFault(n)%fVertex(nv)%Normal(0:1) + &
	               Tdomain%sFault(n)%fFace(j)%normal(0,0:1)
	     nv = Tdomain%sFault(n)%fFace(j)%Face_To_Vertex(1)
	     Tdomain%sFault(n)%fVertex(nv)%Normal(0:1) = Tdomain%sFault(n)%fVertex(nv)%Normal(0:1) + &
	               Tdomain%sFault(n)%fFace(j)%normal(ngll-1,0:1)
	 enddo

         do j = 0, Tdomain%sFault(n)%n_vertex-1
	    normalization = Tdomain%sFault(n)%fVertex(j)%normal(0)**2+Tdomain%sFault(n)%fVertex(j)%normal(1)**2
	    normalization = sqrt(normalization)
	    Tdomain%sFault(n)%fVertex(j)%normal(:) = Tdomain%sFault(n)%fVertex(j)%normal(:)/normalization
	 enddo;

	 do j = 0, Tdomain%sFault(n)%n_face-1
	    ngll = Tdomain%sFault(n)%fFace(j)%ngll
	    allocate (Store_normal(1:ngll-2,0:1))
	    Store_normal (1:ngll-2,0:1) = Tdomain%sFault(n)%fFace(j)%normal(1:ngll-2,0:1)
	    deallocate(Tdomain%sFault(n)%fFace(j)%normal)
	    allocate(Tdomain%sFault(n)%fFace(j)%normal(1:ngll-2,0:1))
	    Tdomain%sFault(n)%fFace(j)%normal(1:ngll-2,0:1) = Store_normal (1:ngll-2,0:1)
	    deallocate (Store_normal)
	 enddo   
	 
    enddo
endif
return
end subroutine shape4

