module svertices

! Modified by Gaetano 01/06/05

type :: vertex

logical :: PML, Abs, FPML
integer :: Glob_Numbering, mat_index

real :: MassMat
real, dimension (:), pointer :: DumpMass, DumpVx, DumpVz, Forces1, Forces2, Veloc1, Veloc2
real, dimension (:), pointer :: Displ, Veloc, Forces, Accel, V0
real, dimension (:), pointer :: Ivx, Ivz,Iveloc1, Iveloc2
real, dimension  (:), pointer :: Double_Value

end type

contains

! ############################################################
subroutine Prediction_Vertex_Veloc (V,alpha,bega, dt)
implicit none

type (Vertex), intent (INOUT) :: V
real, intent (IN) :: bega, dt, alpha
  
     V%Forces = V%Displ + dt * V%Veloc + dt**2 * (0.5 - bega) * V%Accel
     V%V0 = V%Veloc
     V%Forces = alpha * V%Forces + (1-alpha) * V%Displ

return
end subroutine Prediction_Vertex_Veloc

! ###########################################################
subroutine Correction_Vertex_Veloc (V, bega, gam1,dt)
implicit none

type (Vertex), intent (INOUT) :: V
real, intent (IN) :: bega, gam1, dt

integer :: i

      do i = 0,1
          V%Forces(i) = V%MassMat  * V%Forces(i)
      enddo
      if (V%Abs) V%Forces = 0
      V%Veloc  = V%v0+ dt * V%Forces
      V%Accel  = V%Accel + gam1 /dt * (V%Veloc-V%V0)
      V%Displ  =  V%Displ + bega * dt * (V%Veloc+V%V0)
return
end subroutine Correction_Vertex_Veloc

! ###########################################################
subroutine Correction_Vertex_PML_Veloc (V, dt)
implicit none

type (Vertex), intent (INOUT) :: V
real, intent (IN) ::  dt

integer :: i 

if  (V%Abs) then
  V%Veloc1 = 0; V%Veloc2 = 0; V%Veloc = 0
else
  V%V0 = V%Veloc
   do i = 0,1
           V%Veloc1(i) = V%DumpVx(0) * V%Veloc1(i) + dt * V%DumpVx(1)*V%Forces1(i)
           V%Veloc2(i) =V%DumpVz(0) * V%Veloc2(i) + dt * V%DumpVz(1)*V%Forces2(i)
   enddo
  V%Veloc = V%Veloc1 + V%Veloc2
endif

return
end subroutine Correction_Vertex_PML_Veloc
! ###########################################################
subroutine Correction_Vertex_FPML_Veloc (V, dt, fil)
implicit none

type (Vertex), intent (INOUT) :: V
real, intent (IN) ::  dt, fil

integer :: i 
real :: fil2
real :: Ausiliar_Velocity

fil2 = fil**2
V%V0 = V%Veloc

if  (V%Abs) then
  V%Veloc1 = 0; V%Veloc2 = 0; V%Veloc = 0
else

  do i = 0,1
     Ausiliar_Velocity = V%Veloc1(i)       
      V%Veloc1(i) = V%DumpVx(0) * V%Veloc1(i) + dt * &
	    V%DumpVx(1)*V%Forces1(i) + V%Ivx(0) * V%Iveloc1(i)
      V%Iveloc1(i) = Fil2 * V%Iveloc1(i) + 0.5 * (1.-Fil2) *  & 
            (Ausiliar_Velocity + V%Veloc1(i) ) 

    Ausiliar_Velocity = V%Veloc2(i)       	    
    V%Veloc2(i) =V%DumpVz(0) * V%Veloc2(i) + Dt * &
	    V%DumpVz(1)*V%Forces2(i) + V%Ivz(0) * V%IVeloc2(i) 
    V%Iveloc2(i) = Fil2 * V%Iveloc2(i) + 0.5 * (1.-Fil2) * & 
            (Ausiliar_Velocity + V%Veloc2(i) ) 
  enddo
  V%Veloc = V%Veloc1 + V%Veloc2
endif

return
end subroutine Correction_Vertex_FPML_Veloc
! ###########################################################

subroutine get_vfree_vertex(V,Vfree,logic)
implicit none

type (Vertex), intent (IN) :: V
real, dimension (0:1), intent (INOUT) :: Vfree
logical, intent (IN) :: logic 

if (logic) then
        Vfree(0:1) = Vfree(0:1) -  V%MassMat*V%Forces(0:1)
else
	 Vfree(0:1) =  V%MassMat*V%Forces(0:1)
endif     
return
end subroutine get_vfree_vertex


end module svertices

