subroutine Newmark (Tdomain)

! Gaetano Festa, modified 01/06/2005

use sdomain

implicit none
type (domain), intent (INOUT) :: Tdomain

! local variables
integer :: ns, ncc,i,j,n,np, ngllx, ngllz, mat, nelem,nf, w_face, nv_aus, nf_aus, nv
real :: bega, gam1,alpha,dt

real, dimension (0:1) :: V_free_vertex
real, dimension (:,:), allocatable :: Vxloc, Vzloc, V_free

! Predictor-MultiCorrector Newmark Velocity Scheme within a
! Time staggered Stress-Velocity formulation inside PML

! Predictor Phase

if (Tdomain%TimeD%velocity_scheme) then

  alpha =Tdomain%TimeD%alpha
  bega = Tdomain%TimeD%beta / Tdomain%TimeD%gamma
  gam1 = 1. / Tdomain%TimeD%gamma

do n = 0, Tdomain%n_elem -1
     mat = Tdomain%specel(n)%mat_index 
     Dt = Tdomain%sSubDomain(mat)%dt
     
     if (.not. Tdomain%specel(n)%PML) then
           call Prediction_Elem_Veloc (Tdomain%specel(n),alpha, bega, dt)
     else 
           ngllx = Tdomain%specel(n)%ngllx; ngllz = Tdomain%specel(n)%ngllz
           allocate (Vxloc(0:ngllx-1, 0:ngllz-1))
           allocate (Vzloc(0:ngllx-1, 0:ngllz-1))
           call get_PMLprediction_fv2el (Tdomain,n,Vxloc,vzloc,ngllx,ngllz,alpha, bega,dt)
	   if (Tdomain%specel(n)%FPML) then
	        call Prediction_Elem_FPML_Veloc  (Tdomain%specel(n),alpha, bega, dt,Vxloc,Vzloc, & 
	                Tdomain%sSubDomain(mat)%hPrimez, Tdomain%sSubDomain(mat)%hTPrimex, &
			  Tdomain%sSubDomain(mat)%freq)
	   else
                call Prediction_Elem_PML_Veloc  (Tdomain%specel(n),alpha, bega, dt,Vxloc,Vzloc, & 
	                Tdomain%sSubDomain(mat)%hPrimez, Tdomain%sSubDomain(mat)%hTPrimex)
	   endif 
           deallocate (VxLoc, Vzloc)
     endif
enddo

do n = 0, Tdomain%n_face-1 
      mat = Tdomain%sFace(n)%mat_index
      dt = Tdomain%sSubdomain(mat)%dt
      if (.not. Tdomain%sFace(n)%PML)  call Prediction_Face_Veloc (Tdomain%sFace(n),alpha, bega, dt)
enddo     

do n= 0, Tdomain%n_vertex-1
      mat = Tdomain%sVertex(n)%mat_index
      dt = Tdomain%sSubdomain(mat)%dt
      if (.not. Tdomain%sVertex(n)%PML)  call Prediction_Vertex_Veloc (Tdomain%sVertex(n),alpha, bega, dt)
enddo

! Solution phase
do n = 0,Tdomain%n_elem-1
  mat = Tdomain%specel(n)%mat_index
  if (.not. Tdomain%specel(n)%PML ) then
     call get_Displ_fv2el (Tdomain,n)
     call compute_InternalForces_Elem (Tdomain%specel(n), Tdomain%sSubDomain(mat)%hprimex,  &
                    Tdomain%sSubDomain(mat)%hTprimex, Tdomain%sSubDomain(mat)%hprimez, Tdomain%sSubDomain(mat)%hTprimez)
  else
     call compute_InternalForces_PML_Elem (Tdomain%specel(n),Tdomain%sSubDomain(mat)%hprimex, Tdomain%sSubDomain(mat)%hTprimez)
  endif
enddo

! External Forces 
do n = 0, Tdomain%n_source-1
 do ns =0, Tdomain%sSource(n)%ine-1
   ncc = Tdomain%sSource(n)%Elem(ns)%nr
   ngllx = Tdomain%specel(ncc)%ngllx; ngllz = Tdomain%specel(ncc)%ngllz

   if (Tdomain%sSource(n)%i_type_source == 1) then
      do j = 0,ngllz-1
         do i = 0,ngllx-1
            do np = 0,1
               Tdomain%specel(ncc)%Forces(i,j,np) = Tdomain%specel(ncc)%Forces(i,j,np) +   &
                CompSource (Tdomain%sSource(n),Tdomain%TimeD%rtime,np)*  Tdomain%sSource(n)%Elem(ns)%ExtForce(i,j)
            enddo
        enddo
     enddo

   else if (Tdomain%sSource(n)%i_type_source == 2 ) then
      do j = 0,ngllz-1
         do i = 0,ngllx-1
            do np = 0,1
                Tdomain%specel(ncc)%Forces(i,j,np) =Tdomain%specel(ncc)%Forces(i,j,np) +   &
                     CompSource(Tdomain%sSource(n), Tdomain%TimeD%rtime,np)* Tdomain%sSource(n)%Elem(ns)%Explosion(i,j,np)
           enddo  
         enddo  
      enddo  
  endif  
 enddo
enddo

! Communication of Forces

do nf = 0, Tdomain%n_face-1
   Tdomain%sFace(nf)%Forces = 0
   if  (Tdomain%sFace(nf)%PML) then
          Tdomain%sFace(nf)%Forces1 = 0; Tdomain%sFace(nf)%Forces2 = 0
   endif
   nelem = Tdomain%sFace(nf)%Near_element(0)
   w_face = Tdomain%sFace(nf)%Which_face(0)
   call getInternalF_el2f (Tdomain,nelem,nf,w_face,.true.) 
   if (Tdomain%sFace(nf)%PML ) call getInternalF_PML_el2f (Tdomain,nelem,nf,w_face,.true.) 
   nelem = Tdomain%sFace(nf)%Near_element(1)
   if (nelem > -1) then
      w_face = Tdomain%sFace(nf)%Which_face(1)
      call getInternalF_el2f (Tdomain,nelem,nf,w_face,Tdomain%sFace(nf)%coherency) 
      if (Tdomain%sFace(nf)%PML ) call getInternalF_PML_el2f (Tdomain,nelem,nf,w_face,Tdomain%sFace(nf)%coherency) 
   endif
enddo

do nv = 0, Tdomain%n_vertex-1
   Tdomain%sVertex(nv)%Forces= 0
    if (Tdomain%sVertex(nv)%PML ) then
          Tdomain%sVertex(nv)%Forces1= 0;    Tdomain%sVertex(nv)%Forces2= 0
   endif
enddo
do n = 0, Tdomain%n_elem-1
   ngllx= Tdomain%specel(n)%ngllx;     ngllz= Tdomain%specel(n)%ngllz
   nv = Tdomain%specel(n)%Near_Vertex(0)
   Tdomain%sVertex(nv)%Forces(0:1) = Tdomain%sVertex(nv)%Forces(0:1) + Tdomain%specel(n)%Forces(0,0,0:1) 
   if (Tdomain%sVertex(nv)%PML) then
       Tdomain%sVertex(nv)%Forces1(0:1) = Tdomain%sVertex(nv)%Forces1(0:1) + Tdomain%specel(n)%Forces1(0,0,0:1) 
       Tdomain%sVertex(nv)%Forces2(0:1) = Tdomain%sVertex(nv)%Forces2(0:1) + Tdomain%specel(n)%Forces2(0,0,0:1) 
  endif
   nv = Tdomain%specel(n)%Near_Vertex(1)
   Tdomain%sVertex(nv)%Forces(0:1) = Tdomain%sVertex(nv)%Forces(0:1) + Tdomain%specel(n)%Forces(ngllx-1,0,0:1) 
   if (Tdomain%sVertex(nv)%PML) then
       Tdomain%sVertex(nv)%Forces1(0:1) = Tdomain%sVertex(nv)%Forces1(0:1) + Tdomain%specel(n)%Forces1(ngllx-1,0,0:1) 
       Tdomain%sVertex(nv)%Forces2(0:1) = Tdomain%sVertex(nv)%Forces2(0:1) + Tdomain%specel(n)%Forces2(ngllx-1,0,0:1) 
  endif
   nv = Tdomain%specel(n)%Near_Vertex(2)
   Tdomain%sVertex(nv)%Forces(0:1) = Tdomain%sVertex(nv)%Forces(0:1) + Tdomain%specel(n)%Forces(ngllx-1,ngllz-1,0:1) 
   if (Tdomain%sVertex(nv)%PML) then
       Tdomain%sVertex(nv)%Forces1(0:1) = Tdomain%sVertex(nv)%Forces1(0:1) + Tdomain%specel(n)%Forces1(ngllx-1,ngllz-1,0:1) 
       Tdomain%sVertex(nv)%Forces2(0:1) = Tdomain%sVertex(nv)%Forces2(0:1) + Tdomain%specel(n)%Forces2(ngllx-1,ngllz-1,0:1) 
  endif
  nv = Tdomain%specel(n)%Near_Vertex(3)
   Tdomain%sVertex(nv)%Forces(0:1) = Tdomain%sVertex(nv)%Forces(0:1) + Tdomain%specel(n)%Forces(0,ngllz-1,0:1) 
   if (Tdomain%sVertex(nv)%PML) then
       Tdomain%sVertex(nv)%Forces1(0:1) = Tdomain%sVertex(nv)%Forces1(0:1) + Tdomain%specel(n)%Forces1(0,ngllz-1,0:1)
       Tdomain%sVertex(nv)%Forces2(0:1) = Tdomain%sVertex(nv)%Forces2(0:1) + Tdomain%specel(n)%Forces2(0,ngllz-1,0:1)
  endif
enddo


! Definition of a super object

if (Tdomain%logicD%super_object) then
   do n = 0, Tdomain%n_fault-1
        do nf = 0, Tdomain%sFault(n)%n_face-1
	      ngllx = Tdomain%sFault(n)%fFace(nf)%ngll
	      mat = Tdomain%sFault(n)%fFace(nf)%mat_index
	      allocate (V_free(1:ngllx-2,0:1))
	           nf_aus = Tdomain%sFault(n)%fFace(nf)%Face_UP
		   call get_vfree_face(Tdomain%sFace(nf_aus),V_free,ngllx,.false.,.true.)
		   nf_aus = Tdomain%sFault(n)%fFace(nf)%Face_DOWN
		   call get_vfree_face(Tdomain%sFace(nf_aus),V_free,ngllx,.true.,Tdomain%sFault(n)%fFace(nf)%Coherency)
		   select case (Tdomain%sFault(n)%Problem_type)
		     case (1)
                       call compute_traction_on_face_sw (Tdomain%sFault(n)%fFace(nf), V_free,Tdomain%sSubdomain(mat)%dt, &
		                  Tdomain%sFault(n)%imposed_Tolerance)
		     case (2)
		       call compute_traction_on_face_adhesion (Tdomain%sFault(n)%fFace(nf), V_free,Tdomain%sSubdomain(mat)%dt, &
		                  Tdomain%sFault(n)%imposed_Tolerance)
		   end select
	      deallocate (V_free)
	enddo

        do nf = 0, Tdomain%sFault(n)%n_vertex-1
	  if (.not. Tdomain%sFault(n)%fVertex(nf)%Termination) then
	      nv_aus = Tdomain%sFault(n)%fVertex(nf)%Vertex_UP
	      call get_vfree_vertex (Tdomain%sVertex(nv_aus), V_free_Vertex, .false.)
	      nv_aus = Tdomain%sFault(n)%fVertex(nf)%Vertex_DOWN
	      call get_vfree_vertex (Tdomain%sVertex(nv_aus), V_free_Vertex, .true.)
	      select case (Tdomain%sFault(n)%Problem_type)
	        case (1)
                     call compute_traction_on_vertex_sw (Tdomain%sFault(n)%fVertex(nf),    &
		                    V_free_vertex,Tdomain%sSubdomain(mat)%dt, Tdomain%sFault(n)%imposed_Tolerance)

		case (2)
                  call compute_traction_on_vertex_adhesion (Tdomain%sFault(n)%fVertex(nf),   &
		                    V_free_vertex,Tdomain%sSubdomain(mat)%dt, Tdomain%sFault(n)%imposed_Tolerance)
              end select
          endif
	enddo
   enddo

  do  n = 0, Tdomain%n_fault-1

       do nf = 0, Tdomain%sFault(n)%n_face-1
              call rotate_traction_face (Tdomain%sFault(n)%fFace(nf))
       enddo
       do nf = 0, Tdomain%sFault(n)%n_vertex-1
              call rotate_traction_vertex (Tdomain%sFault(n)%fVertex(nf))
        enddo

        do nf = 0, Tdomain%sFault(n)%n_face-1
	       ngllx = Tdomain%sFault(n)%fFace(nf)%ngll
	       nf_aus = Tdomain%sFault(n)%fFace(nf)%Face_UP
	       do i = 0,1
                 Tdomain%sFace(nf_aus)%Forces(1:ngllx-2,i) = Tdomain%sFace(nf_aus)%Forces(1:ngllx-2,i) + &
  	            Tdomain%sFault(n)%fFace(nf)%Bt(1:ngllx-2) * Tdomain%sFault(n)%fFace(nf)%Traction(1:ngllx-2,i)
	       enddo
  	       nf_aus = Tdomain%sFault(n)%fFace(nf)%Face_DOWN
	       if (Tdomain%sFault(n)%fFace(nf)%Coherency) then
	         do i =0,1
                    Tdomain%sFace(nf_aus)%Forces(1:ngllx-2,i) = Tdomain%sFace(nf_aus)%Forces(1:ngllx-2,i) - &
  	              Tdomain%sFault(n)%fFace(nf)%Bt(1:ngllx-2) * Tdomain%sFault(n)%fFace(nf)%Traction(1:ngllx-2,i)
	         enddo
               else
	         do i =0,1
		    do j = 1,ngllx-2
		      Tdomain%sFace(nf_aus)%Forces(j,i) = Tdomain%sFace(nf_aus)%Forces(j,i) - &
  	                 Tdomain%sFault(n)%fFace(nf)%Bt(ngllx-1-j) * Tdomain%sFault(n)%fFace(nf)%Traction(ngllx-1-j,i)
	            enddo
		 enddo
	       endif
        enddo

	do nf = 0, Tdomain%sFault(n)%n_vertex-1
 	   if (.not. Tdomain%sFault(n)%fVertex(nf)%Termination) then
	      nv_aus = Tdomain%sFault(n)%fVertex(nf)%Vertex_UP
	      Tdomain%sVertex(nv_aus)%Forces(0:1) = Tdomain%sVertex(nv_aus)%Forces(0:1) + &
	              Tdomain%sFault(n)%fVertex(nf)%Bt * Tdomain%sFault(n)%fVertex(nf)%Traction(0:1)
	      nv_aus = Tdomain%sFault(n)%fVertex(nf)%Vertex_DOWN
	      Tdomain%sVertex(nv_aus)%Forces(0:1) = Tdomain%sVertex(nv_aus)%Forces(0:1) - &  
	              Tdomain%sFault(n)%fVertex(nf)%Bt * Tdomain%sFault(n)%fVertex(nf)%Traction(0:1)
	   endif
        enddo
   enddo
endif

do n = 0,Tdomain%n_elem-1
    mat = Tdomain%specel(n)%mat_index
    if (.not. Tdomain%specel(n)%PML) then
       call Correction_Elem_Veloc (Tdomain%specel(n),bega, gam1,Tdomain%sSubDomain(mat)%Dt)
   else 
       if (Tdomain%specel(n)%FPML) then
            call Correction_Elem_FPML_Veloc (Tdomain%specel(n),Tdomain%sSubDomain(mat)%Dt, Tdomain%sSubdomain(mat)%freq)
       else
            call Correction_Elem_PML_Veloc (Tdomain%specel(n),Tdomain%sSubDomain(mat)%Dt)
       endif
   endif
enddo

do nf = 0, Tdomain%n_face-1
      mat = Tdomain%sFace(nf)%mat_index 
      if (.not. Tdomain%sFace(nf)%PML) then
          call Correction_Face_Veloc (Tdomain%sFace(nf),bega, gam1,Tdomain%sSubDomain(mat)%Dt)
      else 
          if (Tdomain%sFace(nf)%FPML) then
	     call Correction_Face_FPML_Veloc (Tdomain%sFace(nf),Tdomain%sSubDomain(mat)%Dt, Tdomain%sSubDomain(mat)%freq)
	  else
             call Correction_Face_PML_Veloc (Tdomain%sFace(nf),Tdomain%sSubDomain(mat)%Dt)
	  endif
     endif
enddo

do nv= 0, Tdomain%n_vertex-1
      mat = Tdomain%sVertex(nv)%mat_index 
      if (.not. Tdomain%sVertex(nv)%PML) then
          call Correction_Vertex_Veloc (Tdomain%sVertex(nv),bega, gam1,Tdomain%sSubDomain(mat)%Dt)
      else 
          if (Tdomain%sVertex(nv)%FPML) then
             call Correction_Vertex_FPML_Veloc (Tdomain%sVertex(nv),Tdomain%sSubDomain(mat)%Dt, Tdomain%sSubdomain(mat)%freq)	  
	  else
             call Correction_Vertex_PML_Veloc (Tdomain%sVertex(nv),Tdomain%sSubDomain(mat)%Dt)
          endif
     endif
enddo

endif   ! if Velocity Scheme 
return
end subroutine Newmark
