program  drive_sem

! 2DSPEC, a program written by Gaetano Festa & Elise Delavaud
! Current version 2.0
! Actual revision for publication

! Begin correction in the programming                        19/10/2006
! End correction in the programming 




use sdomain

implicit none
type (domain), target  :: Tdomain
integer :: ntime,i_snap

! ##############  Begin of the program  #######################

write (*,*) "Define mesh properties"
call read_input (Tdomain)

write (*,*) "Compute Gauss-Lobatto-Legendre weights and zeroes"
call compute_GLL (Tdomain)

write (*,*) "Define a global numbering for the collocation points"
call global_numbering (Tdomain)

write (*,*) "Computing shape functions within thier derivatives"
if  (Tdomain%n_nodes == 4) then
       call shape4(TDomain)   ! Linear interpolation
else if (Tdomain%n_nodes == 8) then
       call shape8(TDomain)  ! Quadratic interpolation
else
       write (*,*) " Bad number of nodes for hexaedral shape "
       stop
endif

if (Tdomain%logicD%plot_grid) then
     write (*,*) "Plotting the element mesh"
     call plot_grid (Tdomain)
endif

write (*,*) " Compute Courant parameter"
call compute_Courant (Tdomain)

write (*,*) "Attribute PML properties"
call PML_definition (Tdomain)
! Manual compiling is here
if (Tdomain%logicD%any_source) then
  write (*,*) , "Computing point-source parameters and location"
   call SourcePosition(Tdomain)
endif

if (Tdomain%logicD%save_trace ) then
    write (*,*) "Computing receivers parameters and locations"
    call ReceiverPosition(Tdomain)
endif

if (Tdomain%logicD%super_object) then
   write(*,*) "Define Fault properties"
   call define_fault_properties (Tdomain)
endif

write (*,*) " Allocate fields"
call allocate_domain (Tdomain)

write (*,*) " Compute mass matrix and internal forces coefficients"
call define_arrays (Tdomain)
write (*,*) " Entering the evolution"


Tdomain%TimeD%rtime = 0
if (Tdomain%logicD%save_snapshots .or. Tdomain%logicD%save_deformation)  & 
         Tdomain%timeD%nsnap = Tdomain%TimeD%time_snapshots / Tdomain%TimeD%dtmin
i_snap =1;

do ntime= 0, Tdomain%TimeD%NtimeMax-1
    call Newmark (Tdomain)

    Tdomain%TimeD%rtime = Tdomain%TimeD%rtime + Tdomain%TimeD%dtmin
    if (Tdomain%logicD%save_snapshots .or. Tdomain%logicD%save_deformation)  i_snap = mod (ntime+1, Tdomain%TimeD%nsnap)
    if (i_snap == 0 ) then
         write (*,*) "Iteration number    ", ntime
          if (Tdomain%logicD%save_snapshots) call savefield (Tdomain,ntime)
	  if (Tdomain%logicD%save_deformation) then
	                   call save_vorticity (Tdomain,ntime)
			   call save_deformation (Tdomain, ntime)
	  endif
	  if (Tdomain%logicD%save_fault_trace) call save_fault_trace (Tdomain, ntime)
    endif
    if (Tdomain%logicD%save_trace) call save_trace(Tdomain, ntime)
enddo
! Verification is here

if (Tdomain%logicD%save_trace) then
	write (*,*) "Dumping the records at the receivers"
        call dump_trace(Tdomain)
endif

write (*,*) "The program ended successfully"
 stop
 end program
 
