subroutine shape8(Tdomain)

use sdomain

implicit none

type(domain),target, intent (INOUT) :: Tdomain


! local variables

integer :: i_aus,n, mat,ngllx,ngllz,i,j,ipoint, nv, nv2, Face_UP, ngll

real :: x0,x1,x2,x3,z0,z1,z2,z3,xi,eta,xp,zp, Jac
real :: x4,x5,x6,x7,z4,z5,z6,z7
real :: normal_0, normal_1, normalization, ds_local
real, dimension (0:1,0:1) :: LocInvGrad
real, dimension (:,:), allocatable :: Store_normal
real, dimension (:), pointer :: GLLc_face

! Modified by Gaetano Festa, 27/05/05
!---------------------------------------------------------------------------------------------------------------
! Shape functions are derived from "Finite Elements and Approximations"
! by Zienkiewicz, O.C. and Morgan, K.
! John Wiley and Sons, 1983
! --------------------------------------------------------------------------------------------------------------

allocate (Tdomain%GlobCoord(0:1,0:Tdomain%n_glob_points-1))

do n = 0,Tdomain%n_elem - 1
   i_aus = Tdomain%specel(n)%Control_Nodes(0);  x0 = Tdomain%Coord_Nodes(0,i_aus);  z0 = Tdomain%Coord_Nodes(1,i_aus)
   i_aus = Tdomain%specel(n)%Control_Nodes(1);  x1 = Tdomain%Coord_Nodes(0,i_aus);  z1 = Tdomain%Coord_Nodes(1,i_aus)
   i_aus = Tdomain%specel(n)%Control_Nodes(2);  x2 = Tdomain%Coord_Nodes(0,i_aus);  z2 = Tdomain%Coord_Nodes(1,i_aus)
   i_aus = Tdomain%specel(n)%Control_Nodes(3);  x3 = Tdomain%Coord_Nodes(0,i_aus);  z3 = Tdomain%Coord_Nodes(1,i_aus)
   i_aus = Tdomain%specel(n)%Control_Nodes(4);  x4 = Tdomain%Coord_Nodes(0,i_aus);  z4 = Tdomain%Coord_Nodes(1,i_aus)
   i_aus = Tdomain%specel(n)%Control_Nodes(5);  x5 = Tdomain%Coord_Nodes(0,i_aus);  z5 = Tdomain%Coord_Nodes(1,i_aus)
   i_aus = Tdomain%specel(n)%Control_Nodes(6);  x6 = Tdomain%Coord_Nodes(0,i_aus);  z6 = Tdomain%Coord_Nodes(1,i_aus)
   i_aus = Tdomain%specel(n)%Control_Nodes(7);  x7 = Tdomain%Coord_Nodes(0,i_aus);  z7 = Tdomain%Coord_Nodes(1,i_aus)

   mat = Tdomain%specel(n)%mat_index
   ngllx = Tdomain%specel(n)%ngllx
   ngllz = Tdomain%specel(n)%ngllz

   allocate (Tdomain%specel(n)%Jacob(0:ngllx-1,0:ngllz-1) ) 
   allocate (Tdomain%specel(n)%InvGrad(0:ngllx-1,0:ngllz-1,0:1,0:1) ) 

    do j = 0,ngllz - 1
       eta =   Tdomain%sSubdomain(mat)%GLLcz(j)

       do i = 0,ngllx - 1
          xi = Tdomain%sSubdomain(mat)%GLLcx(i)
          ipoint = Tdomain%specel(n)%Iglobnum(i,j) 

          xp = 0.25 * ( -x0 * (1.-xi)*(1.-eta)*(1+xi+eta) - x1 * (1.+xi)*(1.-eta)*(1-xi+eta) - x2 * (1.+xi)*(1.+eta)*(1-xi-eta) - & 
	       x3 * (1.-xi)*(1.+eta)*(1+xi-eta) )
          xp = xp + 0.5 * ( x4 * (1.-xi**2)*(1.-eta) + x5 * (1.+xi)*(1.-eta**2) +x6 * (1.-xi**2)*(1.+eta) +  &
	       x7 * (1.-xi)*(1.-eta**2))
          zp = 0.25 * ( -z0 * (1.-xi)*(1.-eta)*(1+xi+eta) - z1 * (1.+xi)*(1.-eta)*(1-xi+eta) - z2 * (1.+xi)*(1.+eta)*(1-xi-eta) - &
	       z3 * (1.-xi)*(1.+eta)*(1+xi-eta) )
          zp = zp + 0.5 * ( z4 * (1.-xi**2)*(1.-eta) + z5 * (1.+xi)*(1.-eta**2) +z6 * (1.-xi**2)*(1.+eta) + & 
	       z7 * (1.-xi)*(1.-eta**2) )

          Tdomain%GlobCoord (0,ipoint) = xp;   Tdomain%GlobCoord (1,ipoint) = zp

!         Computation of the derivative matrix, dx_(ii)/dxi_(jj) 

           LocInvGrad(0,0) = 0.25 * (x0 *(1-eta)*(2*xi+eta) + x1 *(1-eta)*(2*xi-eta) + x2 *(1+eta)*(2*xi+eta)+  &
	                 x3 *(1+eta)*(2*xi-eta)) - x4 * xi*(1-eta) - x6 *xi *(1+eta) + 0.5* (x5-x7)* (1-eta**2)
           LocInvGrad(1,0) = 0.25 * (x0 *(1-xi)*(2*eta+xi) - x1 *(1+xi)*(xi-2*eta) + x2 *(1+xi)*(2*eta+xi)-  &
	                 x3 *(1-xi)*(xi-2*eta)) -  x5 *eta*(1+xi) - x7 *eta *(1-xi) + 0.5* (x6-x4)* (1-xi**2)
           LocInvGrad(0,1) = 0.25 * (z0 *(1-eta)*(2*xi+eta) + z1 *(1-eta)*(2*xi-eta) + z2 *(1+eta)*(2*xi+eta)+  &
	                 z3 *(1+eta)*(2*xi-eta)) -  z4 * xi*(1-eta) - z6 *xi *(1+eta) + 0.5* (z5-z7)* (1-eta**2)
           LocInvGrad(1,1) = 0.25 * (z0 *(1-xi)*(2*eta+xi) - z1 *(1+xi)*(xi-2*eta) + z2 *(1+xi)*(2*eta+xi)-   & 
	                 z3 *(1-xi)*(xi-2*eta)) -  z5 *eta*(1+xi) - z7 *eta *(1-xi) + 0.5* (z6-z4)* (1-xi**2)

           call invert2 (LocInvGrad, Jac ) 

           Tdomain%specel(n)%InvGrad (i,j,0:1,0:1)  = LocInvGrad (0:1,0:1)
 
           Tdomain%specel(n)%Jacob (i,j) = Jac
       enddo
   enddo
enddo

if (Tdomain%logicD%super_object) then
   do n = 0, Tdomain%n_fault-1
        do  j = 0, Tdomain%sFault(n)%n_face-1
	     ngll = Tdomain%sFault(n)%fFace(j)%ngll
	     Face_up = Tdomain%sFault(n)%fFace(j)%Face_UP
	     nv2 = Tdomain%sFace(Face_up)%Near_element(0)
	     mat = Tdomain%specel(nv2)%mat_index

	     if (Tdomain%specel(nv2)%near_face(0) == Face_up) then
                   i_aus = Tdomain%specel(nv2)%Control_Nodes(0);
		   x0 = Tdomain%Coord_Nodes(0,i_aus);  z0 = Tdomain%Coord_Nodes(1,i_aus)
                   i_aus = Tdomain%specel(nv2)%Control_Nodes(1);
		   x1 = Tdomain%Coord_Nodes(0,i_aus);  z1 = Tdomain%Coord_Nodes(1,i_aus)
                   i_aus = Tdomain%specel(nv2)%Control_Nodes(4);
		   x2 = Tdomain%Coord_Nodes(0,i_aus);  z2 = Tdomain%Coord_Nodes(1,i_aus)
        	   GLLc_face => Tdomain%sSubdomain(mat)%GLLcx 

             else if (Tdomain%specel(nv2)%near_face(1) == Face_up) then
	           i_aus = Tdomain%specel(nv2)%Control_Nodes(1);
		   x0 = Tdomain%Coord_Nodes(0,i_aus);  z0 = Tdomain%Coord_Nodes(1,i_aus)
                   i_aus = Tdomain%specel(nv2)%Control_Nodes(2);
		   x1 = Tdomain%Coord_Nodes(0,i_aus);  z1 = Tdomain%Coord_Nodes(1,i_aus)
                   i_aus = Tdomain%specel(nv2)%Control_Nodes(5);
		   x2 = Tdomain%Coord_Nodes(0,i_aus);  z2 = Tdomain%Coord_Nodes(1,i_aus)
        	   GLLc_face => Tdomain%sSubdomain(mat)%GLLcz

             else if (Tdomain%specel(nv2)%near_face(2) == Face_up) then
                   i_aus = Tdomain%specel(nv2)%Control_Nodes(2);
		   x0 = Tdomain%Coord_Nodes(0,i_aus);  z0 = Tdomain%Coord_Nodes(1,i_aus)
                   i_aus = Tdomain%specel(nv2)%Control_Nodes(3);
		   x1 = Tdomain%Coord_Nodes(0,i_aus);  z1 = Tdomain%Coord_Nodes(1,i_aus)
                   i_aus = Tdomain%specel(nv2)%Control_Nodes(6);
		   x2 = Tdomain%Coord_Nodes(0,i_aus);  z2 = Tdomain%Coord_Nodes(1,i_aus)
                   GLLc_face => Tdomain%sSubdomain(mat)%GLLcx 

             else
                   i_aus = Tdomain%specel(nv2)%Control_Nodes(3);
		   x0 = Tdomain%Coord_Nodes(0,i_aus);  z0 = Tdomain%Coord_Nodes(1,i_aus)
                   i_aus = Tdomain%specel(nv2)%Control_Nodes(0);
		   x1 = Tdomain%Coord_Nodes(0,i_aus);  z1 = Tdomain%Coord_Nodes(1,i_aus)
                   i_aus = Tdomain%specel(nv2)%Control_Nodes(7);
		   x2 = Tdomain%Coord_Nodes(0,i_aus);  z2 = Tdomain%Coord_Nodes(1,i_aus)
        	   GLLc_face => Tdomain%sSubdomain(mat)%GLLcz

	    endif

	     allocate (Tdomain%sFault(n)%fFace(j)%ds(0:ngll-1))
	     allocate (Tdomain%sFault(n)%fFace(j)%normal(0:ngll-1,0:1))

	     do i = 0, ngll-1
	         xp = x1 * (GLLc_face(i)+0.5) + x0 * (GLLc_face(i)-0.5) - 2*x2* GLLc_face(i)
		 zp  = z1 * (GLLc_face(i)+0.5) + z0 * (GLLc_face(i)-0.5) - 2*z2* GLLc_face(i)

		 ds_local = xp**2 + zp**2
  	         ds_local = sqrt(ds_local)
	         normal_0 = zp/ds_local; normal_1 = -xp/ds_local
		 Tdomain%sFault(n)%fFace(j)%ds(i) = ds_local

! We define the normal for the Face_Down
		normal_0= -normal_0; normal_1 = -normal_1
		 Tdomain%sFault(n)%fFace(j)%normal(i,0) = normal_0; Tdomain%sFault(n)%fFace(j)%normal(i,1) = normal_1
             enddo
	     Tdomain%sFault(n)%fFace(j)%X_Vertex(0) = x0; Tdomain%sFault(n)%fFace(j)%X_Vertex(1) = x1
	     Tdomain%sFault(n)%fFace(j)%Z_Vertex(0) = z0; Tdomain%sFault(n)%fFace(j)%Z_Vertex(1) = z1
        enddo

         do j = 0, Tdomain%sFault(n)%n_vertex-1
	     Tdomain%sFault(n)%fVertex(j)%normal(0:1) = 0
	 enddo

	 do j = 0, Tdomain%sFault(n)%n_face-1
	     ngll = Tdomain%sFault(n)%fFace(j)%ngll
	     nv = Tdomain%sFault(n)%fFace(j)%Face_To_Vertex(0)
	     Tdomain%sFault(n)%fVertex(nv)%Normal(0:1) = Tdomain%sFault(n)%fVertex(nv)%Normal(0:1) + &
	               Tdomain%sFault(n)%fFace(j)%normal(0,0:1)
	     nv = Tdomain%sFault(n)%fFace(j)%Face_To_Vertex(1)
	     Tdomain%sFault(n)%fVertex(nv)%Normal(0:1) = Tdomain%sFault(n)%fVertex(nv)%Normal(0:1) + &
	               Tdomain%sFault(n)%fFace(j)%normal(ngll-1,0:1)
	 enddo

         do j = 0, Tdomain%sFault(n)%n_vertex-1
	    normalization = Tdomain%sFault(n)%fVertex(j)%normal(0)**2+Tdomain%sFault(n)%fVertex(j)%normal(1)**2
	    normalization = sqrt(normalization)
	    Tdomain%sFault(n)%fVertex(j)%normal(:) = Tdomain%sFault(n)%fVertex(j)%normal(:)/normalization
	 enddo
	 
	 do j = 0, Tdomain%sFault(n)%n_face-1
	    ngll = Tdomain%sFault(n)%fFace(j)%ngll
	    allocate (Store_normal(1:ngll-2,0:1))
	    Store_normal (1:ngll-2,0:1) = Tdomain%sFault(n)%fFace(j)%normal(1:ngll-2,0:1)
	    deallocate(Tdomain%sFault(n)%fFace(j)%normal)
	    allocate(Tdomain%sFault(n)%fFace(j)%normal(1:ngll-2,0:1))
	    Tdomain%sFault(n)%fFace(j)%normal(1:ngll-2,0:1) = Store_normal (1:ngll-2,0:1)
	    deallocate (Store_normal)
	 enddo   
   enddo
endif	 
return
end subroutine shape8

